@@ -1,17 +1,15 @@
 package DBIx::Class::Migration;
 
-our $VERSION = "0.04_7";
-$VERSION = eval $VERSION;
+our $VERSION = "0.001";
 
 use Moose;
 use JSON::XS;
 use File::Copy 'cp';
-use File::Spec::Functions 'catdir', 'catfile', 'updir';
+use File::Spec::Functions 'catdir', 'catfile';
 use File::Path 'mkpath', 'remove_tree';
-use DBIx::Class::Migration::Types 'LoadableClass', 'LoadableDBICSchemaClass';
+use MooseX::Types::LoadableClass 'LoadableClass';
 use Class::Load 'load_class';
 use Devel::PartialDump;
-use SQL::Translator;
 
 has db_sandbox_class => (
   is => 'ro',
@@ -43,7 +41,7 @@ has schema_class => (
   is => 'ro',
   predicate=>'has_schema_class',
   required=>0,
-  isa => LoadableDBICSchemaClass,
+  isa => LoadableClass,
   coerce=>1);
 
 has schema_args => (is=>'ro', isa=>'ArrayRef', lazy_build=>1);
@@ -70,9 +68,7 @@ has target_dir_builder => ( is => 'ro', lazy_build => 1);
   sub _infer_schema_class {
     my $self = shift;
     return $self->has_schema_class ?
-      $self->schema_class : $self->has_schema ? 
-        ref($self->schema) : 
-          die "Can't infer schema class without a --schema or --schema_class";
+      $self->schema_class : ref($self->schema);
   }
 
   sub _build_target_dir_builder {
@@ -88,10 +84,26 @@ has target_dir => (is=>'ro', isa=>'Str', lazy_build=>1);
     shift->target_dir_builder->build;
   }
 
-has dbic_dh_args => (is=>'rw', isa=>'HashRef', lazy_build=>1);
+has dbic_dh_args => (is=>'ro', isa=>'HashRef', lazy_build=>1);
 
   sub _build_dbic_dh_args { +{} }
 
+  sub dbic_dh_handles {
+    qw/
+      prepare_install
+      prepare_upgrade
+      prepare_downgrade
+      install
+      upgrade
+      downgrade/;
+  }
+
+has dbic_dh => (
+  is => 'ro',
+  init_arg => undef,
+  lazy_build => 1,
+  handles => [dbic_dh_handles()]);
+
 has schema_loader_class => (
   is => 'ro',
   default => 'DBIx::Class::Migration::SchemaLoader',
@@ -112,7 +124,7 @@ has dbic_fixture_class => (
   isa => LoadableClass,
   coerce=>1);
 
-has dbic_fixtures_extra_args => ( is=>'ro', isa=>'HashRef', lazy_build=>1);
+has dbic_fixtures_extra_args => ( is=>'ro', lazy_build=>1);
 
   sub _build_dbic_fixtures_extra_args { 
     return +{};
@@ -124,12 +136,6 @@ has deployment_handler_class => (
   isa => LoadableClass,
   coerce=>1);
 
-has extra_schemaloader_args => (is=>'ro', isa=>'HashRef', lazy_build=>1);
-
-  sub _build_extra_schemaloader_args { 
-    return +{};
-  }
-
   sub _infer_database_from_storage {
     return (ref(shift) =~m/DBI::(.+)$/)[0];
   }
@@ -152,59 +158,23 @@ has extra_schemaloader_args => (is=>'ro', isa=>'HashRef', lazy_build=>1);
     }
   }
 
-sub normalized_dbic_dh_args {
-  my $self = shift;
-  my %args = %{$self->dbic_dh_args};
-  unless($args{databases}) {
-    $args{databases} = [_infer_database_from_schema($self->schema)];
-    $self->dbic_dh_args(\%args); ## Cache the normalization for next run
+  sub _build_dbic_dh {
+    my $self = shift;
+    my $databases = $self->dbic_dh_args->{databases} ?
+      delete($self->dbic_dh_args->{databases}) :
+      [_infer_database_from_schema($self->schema)];
+
+    (load_class "SQL::Translator::Producer::$_" ||
+      die "No SQLT Producer for $_") for @$databases;
+
+    $self->deployment_handler_class->new({
+      schema => $self->schema,
+      script_directory => catdir($self->target_dir, 'migrations'),
+      databases => $databases,
+      %{$self->dbic_dh_args},
+    })
   }
 
-  return %{$self->dbic_dh_args};
-}
-
-sub dbic_dh {
-  my ($self, @args) = @_;
-  my %dbic_dh_args = $self->normalized_dbic_dh_args;
-
-  (load_class "SQL::Translator::Producer::$_" ||
-    die "No SQLT Producer for $_") for @{$dbic_dh_args{databases}};
-
-  die "A \$VERSION needs to be specified in your schema class ${\$self->_infer_schema_class}"
-  unless $self->schema->schema_version;
-
-
-  my $dh = $self->deployment_handler_class->new({
-    schema => $self->schema,
-    %dbic_dh_args, @args,
-    script_directory => catdir($self->target_dir, 'migrations'),
-  });
-
-  return $dh;
-}
-
-sub prepare_install { shift->dbic_dh->prepare_install }
-sub prepare_upgrade { shift->dbic_dh->prepare_upgrade }
-sub prepare_downgrade { shift->dbic_dh->prepare_downgrade }
-sub install { shift->dbic_dh->install }
-sub upgrade { shift->dbic_dh->upgrade }
-
-sub downgrade {
-  my ($self, %args) = @_;
-  unless($self->dbic_dh_args->{to_version}) {
-    my $to_version = $self->dbic_dh->schema_version - 1;
-    warn "No to_version is specified, downgrading to version $to_version";
-    $args{to_version} = $to_version;
-  }
-  return $self->dbic_dh(%args)->downgrade;
-}
-
-
-before 'downgrade', sub {
-  my ($self, $args) = @_;
-};
-
-
 sub dump { Devel::PartialDump->new->dump(shift) }
 
 sub version { print "Application version is $VERSION\n" }
@@ -312,14 +282,15 @@ sub prepare {
       $self->target_dir, $previous);
     _copy_from_to($previous_fixtures_conf, $fixture_conf_dir);
   } else {
-    print "There is no current database deployed, so I can't prepare upgrades\n";
+    print "There is not current database deployed, so I can't prepare upgrades\n";
     print "or downgrades\n";
   }
 }
 
 sub drop_tables {
   my $self = shift;
-  my $schema = $self->_schema_from_database;
+  my $schema = $self->schema_loader
+    ->schema_from_database($self->_infer_schema_class);
 
   $schema->storage->with_deferred_fk_checks(sub {
     foreach my $source ($schema->sources) {
@@ -336,7 +307,8 @@ sub drop_tables {
 
 sub delete_table_rows {
   my $self = shift;
-  my $schema = $self->_schema_from_database;
+  my $schema = $self->schema_loader
+    ->schema_from_database($self->_infer_schema_class);
 
   $schema->storage->with_deferred_fk_checks(sub {
     my $txn = $schema->txn_scope_guard;
@@ -359,14 +331,8 @@ sub _prepare_fixture_data_dir {
 
 sub build_dbic_fixtures {
   my $dbic_fixtures = (my $self = shift)->dbic_fixture_class;
-  my $version = $self->dbic_dh->version_storage_is_installed ?
-    $self->dbic_dh->database_version : do {
-      print "Since this database is not versioned, we will assume version ";
-      print "${\$self->dbic_dh->schema_version}\n";
-      $self->dbic_dh->schema_version;
-    };
-
-  my $conf_dir = _prepare_fixture_conf_dir($self->target_dir, $version);
+  my $conf_dir = _prepare_fixture_conf_dir($self->target_dir,
+    $self->dbic_dh->database_version);
 
   print "Reading configurations from $conf_dir\n";
 
@@ -378,19 +344,12 @@ sub build_dbic_fixtures {
   $dbic_fixtures->new($init_args);
 }
 
-sub _schema_from_database {
-  my $self = shift;
-  return $self->schema_loader
-    ->schema_from_database(
-      $self->_infer_schema_class,
-      %{$self->extra_schemaloader_args});
-}
-
 sub dump_named_sets {
   (my $self = shift)->dbic_dh->version_storage_is_installed
-    || print "Target DB is not versioned.  Dump may not be reliable.\n";
+    || die "No Database to dump!";
 
-  my $schema = $self->_schema_from_database;
+  my $schema = $self->schema_loader
+    ->schema_from_database($self->_infer_schema_class);
 
   $self->build_dbic_fixtures->dump_config_sets({
     schema => $schema,
@@ -398,50 +357,30 @@ sub dump_named_sets {
     directory_template => sub {
       my ($fixture, $params, $set) = @_;
       $set =~s/\.json//;
-      my $fixture_conf_dir = catfile($fixture->config_dir,updir,$set);
-      mkpath($fixture_conf_dir)
-        unless -d $fixture_conf_dir;
-      return $fixture_conf_dir;
+      _prepare_fixture_data_dir($self->target_dir,
+        $self->dbic_dh->database_version, $set);
     },
   });
 }
 
 sub dump_all_sets {
   (my $self = shift)->dbic_dh->version_storage_is_installed
-    || print "Target DB is not versioned.  Dump may not be reliable.\n";
+    || die "No Database to dump!";
 
-  my $schema = $self->_schema_from_database;
+  my $schema = $self->schema_loader
+    ->schema_from_database($self->_infer_schema_class);
 
   $self->build_dbic_fixtures->dump_all_config_sets({
     schema => $schema,
     directory_template => sub {
       my ($fixture, $params, $set) = @_;
       $set =~s/\.json//;
-      my $fixture_conf_dir = catfile($fixture->config_dir,updir,$set);
-      mkpath($fixture_conf_dir)
-        unless -d $fixture_conf_dir;
-      return $fixture_conf_dir;
+      _prepare_fixture_data_dir($self->target_dir,
+        $self->dbic_dh->database_version, $set);
     },
   });
 }
 
-sub delete_named_sets {
-  my ($self, @sets) = @_;
-  my $fixtures = $self->build_dbic_fixtures;
-  my @paths = map { catfile($fixtures->config_dir,updir,$_) } @sets;
-  foreach my $path (@paths) {
-    $self->_delete_path($path);
-  }
-}
-
-  sub _delete_path {
-    my ($self, $path) = @_;
-    return unless -d $path;
-    print "Deleting $path \n";
-    $path->rmtree;
-  }
-
-
 sub populate_set_to_schema {
   my ($self, $target_set, $schema) = @_;
   $self->build_dbic_fixtures->populate({
@@ -457,7 +396,8 @@ sub populate {
     || die "No Database to populate!";
 
   my $version = $self->dbic_dh->database_version;
-  my $schema = $self->_schema_from_database;
+  my $schema = $self->schema_loader
+    ->schema_from_database($self->_infer_schema_class);
 
   foreach my $set(@_) {
     my $target_set = _prepare_fixture_data_dir(
@@ -475,32 +415,6 @@ sub make_schema {
       catdir($self->target_dir, 'dumped_db'));
 }
 
-sub diagram {
-  my $self = shift;
-  my $number_tables = scalar $self->schema->sources;
-  my $dimension = int sqrt($number_tables * 13);
-  my $trans = SQL::Translator->new(
-    parser => 'SQL::Translator::Parser::DBIx::Class',
-    parser_args => { package => $self->schema },
-    producer => 'GraphViz',
-    producer_args => {
-      skip_tables => 'dbix_class_deploymenthandler_versions',
-      width => $dimension,
-      height => $dimension,
-      show_constraints => 1,
-      show_datatypes => 1,
-      show_sizes => 1,
-      out_file  => $self->_diagram_default_outfile });
-
-  $trans->translate
-    or die $trans->error;
-}
-
-  sub _diagram_default_outfile {
-    my $self = shift;
-    catfile $self->target_dir, 'db-diagram-v' . $self->dbic_dh->schema_version . '.png';
-  }
-
 sub install_if_needed {
   my ($self, %callbacks) = @_;
   if(!$self->dbic_dh->version_storage_is_installed) {
@@ -513,37 +427,11 @@ sub install_if_needed {
   }
 }
 
-sub install_version_storage {
-  my $self = shift;
-  if(!$self->dbic_dh->version_storage_is_installed) {
-    $self->dbic_dh->install_version_storage;
-    $self->dbic_dh->add_database_version({ version => $self->dbic_dh->schema_version });
-    print "Version storage has been installed in the target database\n";
-  } else {
-    print "Version storage is already installed in the target database!\n";
-  }
-}
-
-before [qw/install upgrade downgrade/], sub {
-  my ($self, @args) = @_;
-  %ENV = (
-    %ENV,
-    DBIC_MIGRATION_FIXTURES_OBJ => $self->build_dbic_fixtures,
-    DBIC_MIGRATION_SCHEMA_CLASS => $self->schema_class,
-    DBIC_MIGRATION_TARGET_DIR => $self->target_dir,
-    DBIC_MIGRATION_FIXTURE_DIR => catdir($self->target_dir, 'fixtures', $self->dbic_dh->schema_version),
-    DBIC_MIGRATION_SCHEMA_VERSION => $self->dbic_dh->schema_version,
-    DBIC_MIGRATION_TO_VERSION => $self->dbic_dh->to_version,
-    DBIC_MIGRATION_DATABASE_VERSION => (
-      $self->dbic_dh->version_storage_is_installed ? $self->dbic_dh->database_version : 0),
-  );
-};
-
 __PACKAGE__->meta->make_immutable;
 
 =head1 NAME
 
-DBIx::Class::Migration - Use the best tools together for sane database migrations
+DBIx::Class::Migration - Make database migrations possible
 
 =head1 SYNOPSIS
 
@@ -584,8 +472,6 @@ Utility Commands:
     $migration->delete_table_rows;
     $migration->make_schema;
     $migration->install_if_needed;
-    $migration->install_version_storage;
-    $migration->diagram;
 
 =head1 DESCRIPTION
 
@@ -603,7 +489,7 @@ L<DBIx::Class::DeploymentHandler> and L<DBIx::Class::Fixtures>, along with
 a standard tutorial, to give you a simple and straightforward approach to
 solving the problem of how to best create database versions, migrations and
 testing data.  Additionally it builds on tools like L<Test::mysqld> and
-L<Test::PostgreSQL> along with L<DBD::Sqlite> in order to assist you in quickly
+L<Test::postgresql> along with L<DBD::Sqlite> in order to assist you in quickly
 creating a local development database sandbox.  It offers some integration
 points to testing your database, via tools like L<Test::DBIx::Class> in order to
 make testing your database driven logic less painful.  Lastly, we offer some
@@ -645,7 +531,7 @@ running tests, for example.
 
 =head2 db_sandbox_class
 
-Accepts Str.  Not Required (defaults to 'DBIx::Class::Migration::SqliteSandbox').
+Accepts Str.  Not Required (defaults to 'DBIx::Class::Migration::MySQLSandbox').
 
 Unless you already have a database setup and running (as you probably do in
 production) we need to auto create a database 'sandbox' that is isolated to
@@ -660,7 +546,7 @@ an array which can be sent to L<DBIx::Class::Schema/connect>.
 This defaults to L<DBIx::Class::Migration::SqliteSandbox>.  Currently we have
 support for MySQL and Postgresql via L<DBIx::Class::Migration::MySQLSandbox>
 and L<DBIx::Class::Migration::PgSandbox>, but you will need to side install
-L<Test::mysqld> and L<Test::PostgreSQL> (In other words you'd need to add
+L<Test::mysqld> and L<Test::postgresql> (In other words you'd need to add
 these C<Test::*> namespace modules to your C<Makefile.PL> or C<dist.ini>).
 
 =head2 db_sandbox
@@ -697,7 +583,7 @@ database connection with the following DSN:
 
     DBD:SQLite:[path to target_dir]/[db_file_name].db
 
-Where c<[path to target_dir]> is L</target_dir> and [db_file_name] is a converted
+Where c<[path to home_dir]> is L</target_dir> and [db_file_name] is a converted
 version of L</schema_class>.  For example if you set L<schema_class> to:
 
     MyApp::Schema
@@ -838,14 +724,6 @@ following defaults:
 
 L</dbic_dh_args> will overwrite the defaults, if you pass them.
 
-=head2 extra_schemaloader_args
-
-Optional.  Accepts a HashRef of arguments you can use to influence how
-L<DBIx::Class::Schema::Loader> works.  This HashRef would get passed as
-C<loader_options> (see L<DBIx::Class::Schema::Loader/make_schema_at>.
-
-Meaningful options are described at L<DBIx::Class::Schema::Loader::Base>.
-
 =head1 METHODS
 
 This class defines the following methods for public use
@@ -873,7 +751,7 @@ choose to leave this undefined.
 =head3 Specify a schema
 
 There may be some cases when you already have a schema object constructed and
-would prefer to just use that.  For example, you may be using L<Catalyst> and
+woudl prefer to just use that.  For example, you may be using L<Catalyst> and
 wish to build custom scripts using the built-in dependency and service lookup:
 
     use MyCatalyst::App;
@@ -884,7 +762,7 @@ wish to build custom scripts using the built-in dependency and service lookup:
       %{MyCatalyst::App->config->{extra_migration_init_args}};
     );
 
-Be careful of potential locking issues when using some databases like SQLite.
+Be care of potential locking issues when using some databases like SQLite.
 
 =head3 OPTIONAL: Specify a target_dir
 
@@ -921,8 +799,7 @@ and reflect a desire for long term code flexibility and clean design.
 
 =head2 version
 
-Prints to STDOUT a message regarding the version of L<DBIC:Migration> that you
-are currently running.
+Prints to STDOUT a message regarding the current application $VERSION
 
 =head2 status
 
@@ -933,7 +810,8 @@ of the current C<schema> version.  Sends this as a string to STDOUT
 
 Creates a C<fixtures> and C<migrations> directory under L</target_dir> (if they
 don't already exist) and makes deployment files for the current schema.  If
-deployment files exist, will fail unless you L</overwrite_migrations>. 
+deployment files exist, will fail unless you L</overwrite_migrations> and
+L</overwrite_fixtures>.
 
 The C<migrations> directory reflects a directory structure as documented in
 L<DBIx::Class::DeploymentHandler>.
@@ -947,10 +825,7 @@ safety reasons, we never overwrite any fixture configs.
 
 Installs either the current schema version (if already prepared) or the target
 version specified via any C<to_version> flags sent as an L<dbic_dh_args> to the
-database which is connected via L</schema>.
-
-If you try to install to a database that has already been installed, you'll get
-an error.  See L</drop_tables>.
+database which is connected via L</schema>
 
 =head2 upgrade
 
@@ -998,23 +873,6 @@ the matching sets for that version.
 Skips the table C<dbix_class_deploymenthandler_versions>, so you don't lose
 deployment info (this is different from L</drop_tables> which does delete it.)
 
-=head2 diagram
-
-Experimental feature.  Although not specifically a migration task, I find it
-useful to output visuals of my databases.  This command will place a file in
-your L</target_dir> called C<db-diagram-vXXX.png> where C<XXX> is he current
-C<schema> version.
-
-This is using the Graphviz producer (L<SQL::Translator::Producer::GraphViz>)
-which in turn requires L<Graphviz>.  Since this is not always trivial to
-install, I do not require it.  You will need to add it manually to your
-C<Makefile.PL> or C<dist.ini> and manage it yourself.
-
-This feature is experimental and currently does not offer any options, as I
-am still determining the best way to meet the need without exceeding the
-scope of L<DBIx::Class::Migration>.  Consider this command a 'freebee' and
-please don't depend on it in your custom code.
-
 =head2 install_if_needed
 
 If the database is not installed, do so.  Accepts a hash of callbacks or
@@ -1043,49 +901,10 @@ object).
 
 Accepts: Arrayref of fixture sets
 
-    $migration->install_if_needed(
-      default_fixture_sets => ['all_tables']);
-
 After database installation, populate the fixtures in order.
 
 =back
-
-=head2 install_version_storage
-
-If the targeted (connected) database does not have the versioning tables
-installed, this will install them.  The version is set to whatever your
-C<schema> version currently is.
-
-You will only need to use this command in the case where you have an existing
-database that you are reverse engineering and you need to setup versioning
-storage since you can't rebuild the database from scratch (such as if you have
-a huge production database that you now want to start versioning).
-
-=head2 delete_named_sets
-
-Given a (or a list) of fixture sets, delete them if the exist in the current
-schema version.
-
-Yes, this really deletes, you've been warned (check in your code to a source
-control repository).
-
-=head1 ENVIRONMENT
-
-When running L<DBIx::Class::Migration> we set some C<%ENV> variables during
-installation, up / downgrading, so that your Perl run scripts (see
-L<DBIx::Class::DeploymentHandler::DeployMethod::SQL::Translator\'PERL SCRIPTS'>)
-can receive some useful information.  The Following C<%ENV> variables are set:
-
-    DBIC_MIGRATION_SCHEMA_CLASS => $self->schema_class
-    DBIC_MIGRATION_TARGET_DIR => $self->target_dir
-    DBIC_MIGRATION_FIXTURE_DIR => catdir($self->target_dir, 'fixtures', $self->dbic_dh->schema_version),
-    DBIC_MIGRATION_SCHEMA_VERSION => $self->dbic_dh->schema_version
-    DBIC_MIGRATION_TO_VERSION => $self->dbic_dh->to_version
-    DBIC_MIGRATION_DATABASE_VERSION => $self->dbic_dh->schema_version || 0
-
-You might find having these available in your migration scripts useful for
-doing things like 'populate a database from a fixture set, if it exists, but
-if not run a bunch of inserts.
+Currently we allow one callback C<on_install> which gets passed two arguments:
 
 =head1 THANKS
 
@@ -1104,7 +923,7 @@ sets of data.  Lastly, thanks to the L<DBIx::Class> cabal for all the work done
 in making the L<DBIx::Class> ORM so amazingly powerful.
 
 Additionally thanks to the creators / maintainers for L<Test::mysqld> and
-L<Test::PostgreSQL>, which made it easy to create developer level sandboxes for
+L<Test::postgresql>, which made it easy to create developer level sandboxes for
 these popular open source databases.
 
 As usual, thanks to the L<Moose> cabal for making Perl programming fun and
@@ -1115,36 +934,35 @@ don't need to write my own build and deployment tools.
 
 John Napiorkowski L<email:jjnapiork@cpan.org>
 
-=head1 CONTRIBUTORS
-
-The following is a list of identified contributors.  Please let me know if I
-missed you.
-
-    https://github.com/pjcj
-    https://github.com/chromatic
-    https://github.com/bentglasstube
-    https://github.com/logie17
-    https://github.com/RsrchBoy
-    https://github.com/vkroll
-    https://github.com/felliott
-    https://github.com/mkrull
-    https://github.com/moltar
-    https://github.com/andyjones
-    https://github.com/pnu
-
 =head1 SEE ALSO
 
 L<DBIx::Class::DeploymentHandler>, L<DBIx::Class::Fixtures>, L<DBIx::Class>,
 L<DBIx::Class::Schema::Loader>, L<Moose>, L<DBIx::Class::Migration::Script>,
 L<DBIx::Class::Migration::Population>, L<dbic-migration>, L<SQL::Translator>,
-L<Test::mysqld>, L<Test::PostgreSQL>.
+L<Test::mysqld>, L<Test::postgresql>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2013, John Napiorkowski L<email:jjnapiork@cpan.org>
+Copyright 2012, John Napiorkowski L<email:jjnapiork@cpan.org>
 
 This library is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
 =cut
 
+__END__
+
+TODO
+
+list-fixture-history
+list-migration-history
+delete fixture/migration version
+list-fixture-sets
+?? From version? ??
+Dzil and module install plugins
+shell version
+add ability to exclude some tables from drop/delete
+
+?? patch DH to abstract the filesysteem storage and get methods for 'last/next version'
+?? patch DBIC-deploymenthander for autoversions
+
